package services

import (
	"errors"
	"flag"
)

type DbConfig struct {
	Host   string `json:"host,omitempty" yaml:"host,omitempty"`
	Port   int    `json:"port,omitempty" yaml:"port,omitempty"`
	Schema string `json:"schema,omitempty" yaml:"schema,omitempty"`
	User   string `json:"user,omitempty" yaml:"user,omitempty"`
	Pwd    string `json:"pwd,omitempty" yaml:"pwd,omitempty"`
}

const (
	defCfgFilePath = "conf/db.yaml"
)

var (
	dbConfigInstance *DbConfig
	cfgFilePath      string
)

func init() {
	dbConfigInstance = &DbConfig{}
	flag.StringVar(&dbConfigInstance.Host, "h", "", "database host")
	flag.IntVar(&dbConfigInstance.Port, "p", 0, "database port")
	flag.StringVar(&dbConfigInstance.Schema, "s", "", "database schema")
	flag.StringVar(&dbConfigInstance.User, "u", "", "database user")
	flag.StringVar(&dbConfigInstance.Pwd, "P", "", "database password")
	flag.StringVar(&cfgFilePath, "c", "", "database config file path")
}

func (cfg *DbConfig) IsZero() bool {
	if cfg == nil {
		return true
	}

	if cfg.Host != "" ||
		cfg.Port != 0 ||
		cfg.Schema != "" ||
		cfg.User != "" ||
		cfg.Pwd != "" {
		return false
	}

	return true
}

func (cfg *DbConfig) FixDefaultValues() error {
	if cfg == nil {
		return errors.New("can not fix values of nil object")
	}

	if cfg.Schema == "" {
		return errors.New("can not fix default values with empty database schema")
	}

	if cfg.Pwd == "" {
		return errors.New("can not fix default values with empty database password")
	}

	if cfg.Host == "" {
		cfg.Host = "localhost"
	}

	if cfg.Port == 0 {
		cfg.Port = 3306
	}

	if cfg.User == "" {
		cfg.User = "root"
	}

	return nil
}
